<?php 
namespace WCMC\classes\com;

class Currency
{
	var $table_rates_url = "https://vanquishplugins.com/currency/compute_rate.php?base=";
	var $currency_list_cache;
	function __construct()
	{
		add_action('wp_ajax_wcmc_load_currency_rates', array(&$this, 'ajax_load_currency_rates'));	
		
		//hooks list 
		/*
		woocommerce_currency
		woocommerce_currency_symbols //to inject custom symbols, no need
		woocommerce_currency_symbol */
		add_filter('woocommerce_currency', array(&$this, 'override_woocommerce_currency'));
		add_filter('woocommerce_currency_symbol', array(&$this, 'override_woocommerce_currency_symbol'), 10,2);
		//add_filter('woocommerce_product_get_price', array(&$this, 'override_woocommerce_product_get_price'), 10,2);
		add_filter('raw_woocommerce_price', array(&$this, 'override_woocommerce_product_get_price'));
	}
	private function is_current_page_to_be_exluded()
	{
		global $post, $wp;
		
		if(!$this->exists_base_currency())
			return true;
		
		//Orders list page and ajax request to retrieve order data
		if((wcmc_get_value_if_set($_GET,'post_type', false) == 'shop_order') || (wcmc_get_value_if_set($_GET, 'action', false) == 'woocommerce_get_order_details'))
			return true;
		
		if(isset($post) && $post->post_type == 'shop_order')
			return true;
		//
		
		//WooCommerce settings page
		if(wcmc_get_value_if_set($_GET,'page', false) == 'wc-settings')
			return true;
		
		//Frontend "order-received", "order list" "view-order" endpoints
		$end_points = array(get_option('woocommerce_checkout_order_received_endpoint'), 
							get_option('woocommerce_myaccount_view_order_endpoint'), 
							get_option('woocommerce_myaccount_orders_endpoint'));
		
		if(!is_admin() && is_array($wp->query_vars))
			foreach ( $wp->query_vars as $key => $value ) 
				if(in_array($key, $end_points) /* && $value == "" */)
				{
					/* wcmc_var_dump($wp->query_vars);	
					wcmc_var_dump($key);	
					wcmc_var_dump($end_points);	 */
					return true;	
				}
		//
		
		return false;
		
	}
	public function override_woocommerce_currency($woocommerce_currency)
	{
		global $wcmc_customer_model;
		if($this->is_current_page_to_be_exluded())
			return $woocommerce_currency;
		
		//If admin area, base currency is used
		return is_admin() ? $this->get_base_currency() : $wcmc_customer_model->get_currency();
	}
	public function override_woocommerce_currency_symbol( $currency_symbol, $currency )
	{
		global $wcmc_customer_model;
	
		if($this->is_current_page_to_be_exluded())
			return $currency_symbol;
		
		$currencies_data = $this->get_currencies_list();
		//If admin area, base currency is used
		$current_currency_code = is_admin() ? $this->get_base_currency() : $wcmc_customer_model->get_currency();
		$symbol = wcmc_get_value_if_set( $currencies_data, array($current_currency_code, 'currencySymbol'), " ".$current_currency_code );
		return $symbol;
	}
	public function override_woocommerce_product_get_price($price, $product = null  )
	{
		global $wcmc_customer_model, $wcmc_option_model;
		
		if($this->is_current_page_to_be_exluded() || !isset($price) || !is_numeric($price))
			return $price;
	
		$currencies_data = $this->get_currencies_list();
		//If admin area, base currency is used
		$current_currency_code = is_admin() ? $this->get_base_currency() : $wcmc_customer_model->get_currency();
		$currency_data = $wcmc_option_model->get_currency_options();
		
		$current_rate =  wcmc_get_value_if_set( $currency_data, array('currency_data', $current_currency_code, 'rate'), false);
		
		if($current_rate == false || $current_rate == 0 || !is_numeric($current_rate))
			return $price;
		
		$new_price = $price * $current_rate;
		return wc_format_decimal($new_price);
	}
	public function ajax_load_currency_rates()
	{
		if(isset($_POST['base_currency']))
		{
			$result = $this->update_currency_rates($_POST['base_currency']);
			
			if($result['code'] == 0)
				echo "error";
			else 
				echo json_encode($result['data']['currency_data']);
		}
		wp_die();
	}
	public function get_woocommerce_default_currency()
	{
		return get_option('woocommerce_currency');
	}
	public function exists_base_currency()
	{
		global $wcmc_option_model;
		$options = $wcmc_option_model->get_currency_options();
		
		foreach($options['currency_data'] as $currency_code => $currency_data)
			if(wcmc_get_value_if_set($currency_data, 'base_currency', false))
			{
				return true;
			}
			
		return false;
	}
	public function get_base_currency()
	{
		global $wcmc_option_model;
		$options = $wcmc_option_model->get_currency_options();
		
		foreach($options['currency_data'] as $currency_code => $currency_data)
			//if($currency_data['base_currency'])
			if(wcmc_get_value_if_set($currency_data, 'base_currency', false))
			{
				//wcmc_var_dump($currency_code);
				return $currency_code;
			}
			
		$default_currency = $this->get_woocommerce_default_currency();
		
		//If none has been selected, the default set via WooCommerce is used	
		return $default_currency;
	}
	function is_currency_available($currency_code)
	{ 
		return !isset($currency_code) || $currency_code == "" ? false : array_key_exists($currency_code, $this->get_available_currencies_info());
	}
	function get_default_currency_for_location($location)
	{
		$currencies_info = $this->get_available_currencies_info();
		foreach($currencies_info as $currency_code => $currency_data)
		{
			if(isset($currency_data['default_for_country']) && in_array($location, $currency_data['default_for_country']))
				return $currency_code;
		}
		
		return false;
	}
	function get_available_currencies_info()
	{
		global $wcmc_option_model;
		$currency_options = $wcmc_option_model->get_currency_options(); //rates and base currency
		$currencies_data = $this->get_currencies_list(); //currency list with symbols and name
		$result = array();
		//wcmc_var_dump($currency_options);
		foreach($currency_options['currency_data'] as $currency_code => $currency_data)
		{
			$currencies_data[$currency_code]['flag'] = '<img class="currency-flag currency-flag-'.strtolower($currency_code).' currency-flag-sm"/>';
			//all info are copied from the main source to the final one in which there may be additional info like flags
			foreach($currency_data as $key => $value)
				$currencies_data[$currency_code][$key] = $value;
			$result[$currency_code] = $currencies_data[$currency_code];
		}								
		
		/*
		array(3) {
		  ["BRL"]=>
		  array(5) {
			["currencyName"]=>
			string(14) "Brazilian Real"
			["currencySymbol"]=>
			string(2) "R$"
			["id"]=>
			string(3) "BRL"
			["flag"]=>
			string(63) ""
			["rate"]=>
			string(7) "5.06309"
		  }
		  ["GIP"]=>
		  array(6) {
			["currencyName"]=>
			string(15) "Gibraltar Pound"
			["currencySymbol"]=>
			string(2) "£"
			["id"]=>
			string(3) "GIP"
			["flag"]=>
			string(63) ""
			["rate"]=>
			string(1) "1"
			["base_currency"]=>    //<---------------------- Key exists only for base currency
			string(4) "true",
			["default_for_country"]=>  //<---------------------- Key exists only in at least one country has been selected
			  array(2) {
				[0]=>
				string(2) "DE"
				[1]=>
				string(2) "IT"
			  }
		  }
		  ["INR"]=>
		  array(5) {
			["currencyName"]=>
			string(12) "Indian Rupee"
			["currencySymbol"]=>
			string(3) "₹"
			["id"]=>
			string(3) "INR"
			["flag"]=>
			string(63) ""
			["rate"]=>
			string(8) "90.01448"
		  }
		}
		*/
		
	    return $result;
	}
	public function update_currency_rates($base_currency, $update_settings = false)
	{
		global $wcmc_option_model;
		
		if(!$this->is_currency_available($base_currency))
		{
			return array("code" => 0, "data" => array());
		}
		//1. load from remote the json with rates
		$result = file_get_contents($this->table_rates_url.$base_currency);
		$result = json_decode($result, true);
		if($result == false)
			return array("code" => 0, "data" => array());
		
		//wcmc_var_dump($result['results']);
		
		
		//2. update rates
		$currency_options = $wcmc_option_model->get_currency_options();		
		$currency_options['currency_data'][$base_currency]['rate'] = 1; //base country
		$currency_options['currency_data'][$base_currency]['base_currency'] = true; 
		foreach($result['results'] as $currency_data)
		{
			if(!isset($currency_options['currency_data'][$currency_data['to']]))
				continue;
		
			$currency_options['currency_data'][$currency_data['to']]['rate'] = round ($currency_data['val'], 5);
			$currency_options['currency_data'][$base_currency]['base_currency'] = false; 
		}
		
		//3. optionally settings can be updated
		if($update_settings)
			$wcmc_option_model->save_currency_options($currency_options);
		
		return array("code" => 1, "data" => $currency_options);		
	}
	public function get_currencies_list()
	{
		if(isset($this->currency_list_cache))
			return $this->currency_list_cache;
		
		$currency_to_remove = array(/* 'BYR', 'SDG','TMT', */ 'XDR', 'BTC',/* , 'XOF',  'LSL', 'PAB', 'LVL', 'XAF' */); 
		$result = file_get_contents(WCMC_PLUGIN_ABS_PATH."/assets/currencies.json"); //https://free.currencyconverterapi.com/api/v6/currencies?apiKey=sample-api-key
		$result = json_decode($result, true);
		if(isset($result['results']))
		{
			ksort($result['results']);
			$result = $result['results'];
			foreach($currency_to_remove as $to_remove)
				if(isset($result[$to_remove]))
					unset($result[$to_remove]);
			
		}
		else 
			$result = array();
		
		$this->currency_list_cache = $result;
		
		/* wcmc_var_dump($result); */
		return $result;
	}
}
?>